//******************************************************************************
//
// File:    ChatFrame.java
// Package: edu.rit.chat1
// Unit:    Class edu.rit.chat1.ChatFrame
//
// This Java source file is copyright (C) 2006 by Alan Kaminsky. All rights
// reserved. For further information, contact the author, Alan Kaminsky, at
// ark@cs.rit.edu.
//
// This Java source file is part of the M2MI Library ("The Library"). The
// Library is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// The Library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// A copy of the GNU General Public License is provided in the file gpl.txt. You
// may also obtain a copy of the GNU General Public License on the World Wide
// Web at http://www.gnu.org/licenses/gpl.html or by writing to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//******************************************************************************

package edu.rit.chat1;

import edu.rit.util.Transcript;

import java.awt.Font;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Class ChatFrame encapsulates the UI for a rudimentary M2MI-based chat
 * application.
 *
 * @author  Alan Kaminsky
 * @version 26-Mar-2006
 */
public class ChatFrame
	extends JFrame
	{

// Hidden constants.

	private static final int GAP = 10;

	private static final Font CHAT_FONT = new Font ("sanserif", Font.PLAIN, 12);

// Hidden data members.

	private Transcript myChatLog;
	private JTextField myMessageField;
	private JButton mySendButton;

	private ChatFrameListener myListener;

// Exported constructors.

	/**
	 * Construct a new chat frame.
	 *
	 * @param  title  Title.
	 */
	public ChatFrame
		(String title)
		{
		super (title);

		JPanel theMainPanel = new JPanel();
		add (theMainPanel);
		theMainPanel.setBorder
			(BorderFactory.createEmptyBorder (GAP, GAP, GAP, GAP));
		theMainPanel.setLayout
			(new BoxLayout (theMainPanel, BoxLayout.Y_AXIS));

		myChatLog = new Transcript (CHAT_FONT, 240, 12, 40);
		theMainPanel.add (myChatLog);

		theMainPanel.add (Box.createVerticalStrut (GAP));

		JPanel theEntryPanel = new JPanel();
		theMainPanel.add (theEntryPanel);
		theEntryPanel.setLayout
			(new BoxLayout (theEntryPanel, BoxLayout.X_AXIS));

		myMessageField = new JTextField (40);
		myMessageField.addActionListener (new ActionListener()
			{
			public void actionPerformed
				(ActionEvent e)
				{
				doSend();
				}
			});
		theEntryPanel.add (myMessageField);

		theEntryPanel.add (Box.createHorizontalStrut (GAP));

		mySendButton = new JButton ("Send");
		mySendButton.addActionListener (new ActionListener()
			{
			public void actionPerformed
				(ActionEvent e)
				{
				doSend();
				}
			});
		theEntryPanel.add (mySendButton);

		setDefaultCloseOperation (EXIT_ON_CLOSE);
		pack();
		setVisible (true);
		myMessageField.requestFocusInWindow();
		}

// Exported operations.

	/**
	 * Register the given chat frame listener. Henceforth, whenever the user
	 * sends a line of text in this chat frame, the given chat frame listener's
	 * {@link ChatFrameListener#send(String) send()} method will be called with
	 * the line of text.
	 *
	 * @param  theListener  Chat frame listener.
	 */
	public synchronized void setListener
		(ChatFrameListener theListener)
		{
		myListener = theListener;
		}

	/**
	 * Obtain this chat frame's chat log.
	 *
	 * @return  Chat log (transcript).
	 */
	public synchronized Transcript getChatLog()
		{
		return myChatLog;
		}

// Hidden operations.

	/**
	 * Take action when the Send button is pressed.
	 */
	private synchronized void doSend()
		{
		String line = myMessageField.getText();
		if (line != null && line.length() > 0 && myListener != null)
			{
			myListener.send (line);
			}
		myMessageField.setText ("");
		myMessageField.requestFocusInWindow();
		}

	}
